@@ -42,6 +42,7 @@ typedef enum {
   MODE_TEXTINFO,
   MODE_WARNING,
   MODE_INFO,
+  MODE_NOTIFICATION,
   MODE_ABOUT,
   MODE_LAST
 } ZenityDialogMode;
@@ -64,6 +65,7 @@ typedef struct {
   ZenityProgressData *progress_data;
   ZenityTextData *text_data;
   ZenityTreeData *tree_data;
+  ZenityNotificationData *notification_data;
 } ZenityParsingOptions;
 
 enum {
@@ -79,6 +81,7 @@ enum {
   OPTION_TEXTINFO,
   OPTION_TEXTEDIT,
   OPTION_WARNING,
+  OPTION_NOTIFICATION,
   OPTION_TITLE,
   OPTION_ICON,
   OPTION_WIDTH,
@@ -94,6 +97,8 @@ enum {
   OPTION_INFOTEXT,
   OPTION_FILENAME,
   OPTION_MULTIFILE,
+  OPTION_DIR,
+  OPTION_SAVE,
   OPTION_TEXTFILENAME,
   OPTION_LISTTEXT,
   OPTION_COLUMN,
@@ -105,18 +110,21 @@ enum {
   OPTION_PERCENTAGE,
   OPTION_PULSATE,
   OPTION_AUTOCLOSE,
+  OPTION_PRINTCOLUMN,
   OPTION_QUESTIONTEXT,
   OPTION_WARNINGTEXT,
+  OPTION_NOTIFICATIONICON,
+  OPTION_NOTIFICATIONTEXT,
   OPTION_ABOUT,
   OPTION_VERSION,
   OPTION_LAST,
 };
 
 static void zenity_parse_options_callback (poptContext              ctx,
-    enum poptCallbackReason  reason,
-    const struct poptOption *opt,
-    const char              *arg,
-    void                    *data);
+					   enum poptCallbackReason  reason,
+					   const struct poptOption *opt,
+					   const char              *arg,
+					   void                    *data);
 
 struct poptOption options[] = {
   {
@@ -182,6 +190,15 @@ struct poptOption options[] = {
     N_("Display list dialog"),
     NULL
   },
+  {
+    "notification",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_NOTIFICATION,
+    N_("Display notification"),
+    NULL
+  },
   {
     "progress",
     '\0',
@@ -439,6 +456,24 @@ struct poptOption file_selection_options[] = {
     N_("Allow multiple files to be selected"),
     NULL
   },
+  {
+    "directory",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_DIR,
+    N_("Activate directory-only selection"),
+    NULL
+  },
+  {
+    "save",
+    '\0',
+    POPT_ARG_NONE,
+    NULL,
+    OPTION_SAVE,
+    N_("Activate save mode"),
+    NULL
+  },
   {
     "separator",
     '\0',
@@ -515,6 +550,37 @@ struct poptOption list_options[] = {
     N_("Allow changes to text"),
     NULL
   },
+  {
+    "print-column",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_PRINTCOLUMN,
+    N_("Print a specific column (Default is 1. 'ALL' can be used to print all columns)"),
+    NULL
+  },
+  POPT_TABLEEND
+};
+
+struct poptOption notification_options[] = {
+  {
+    NULL,
+    '\0',
+    POPT_ARG_CALLBACK | POPT_CBFLAG_POST,
+    zenity_parse_options_callback,
+    0,
+    NULL,
+    NULL
+  },
+  {
+    "text",
+    '\0',
+    POPT_ARG_STRING,
+    NULL,
+    OPTION_NOTIFICATIONTEXT,
+    N_("Set the notification text"),
+    NULL
+  },
   POPT_TABLEEND
 };
 
@@ -866,6 +932,15 @@ struct poptOption application_options[] = {
     N_("List options"),
     NULL
   },
+  {
+    NULL,
+    '\0',
+    POPT_ARG_INCLUDE_TABLE,
+    notification_options,
+    0,
+    N_("Notication options"),
+    NULL
+  },
   {
     NULL,
     '\0',
@@ -949,6 +1024,7 @@ zenity_init_parsing_options (void) {
   results->progress_data = g_new0 (ZenityProgressData, 1); 
   results->text_data = g_new0 (ZenityTextData, 1);
   results->tree_data = g_new0 (ZenityTreeData, 1);
+  results->notification_data = g_new0 (ZenityNotificationData, 1);
 
   /* Give some sensible defaults */
   results->data->width = -1;
@@ -961,6 +1037,8 @@ zenity_init_parsing_options (void) {
   results->calendar_data->year = 0;
   results->calendar_data->dialog_text = NULL;
   results->file_data->multi = FALSE;
+  results->file_data->directory = FALSE;
+  results->file_data->save = FALSE;
   results->file_data->separator = g_strdup ("|");
   results->text_data->editable = FALSE;
   results->tree_data->separator = g_strdup ("|");
@@ -972,6 +1050,7 @@ zenity_init_parsing_options (void) {
   results->tree_data->checkbox = FALSE;
   results->tree_data->radiobox = FALSE;
   results->tree_data->editable = FALSE;
+  results->tree_data->print_column = NULL;
 }
 
 static void
@@ -1020,6 +1099,12 @@ zenity_free_parsing_options (void) {
         g_slist_foreach (results->tree_data->columns, (GFunc) g_free, NULL);
       if (results->tree_data->separator)
         g_free (results->tree_data->separator);
+      if (results->tree_data->print_column)
+        g_free (results->tree_data->print_column);
+      break;
+    case MODE_NOTIFICATION:
+      if (results->notification_data->notification_text)
+        g_free (results->notification_data->notification_text);
       break;
     default:
       break;
@@ -1090,6 +1175,9 @@ main (gint argc, gchar **argv) {
       results->tree_data->data = poptGetArgs (ctx);
       zenity_tree (results->data, results->tree_data);
       break;
+    case MODE_NOTIFICATION:
+      zenity_notification (results->data, results->notification_data);
+      break;
     case MODE_PROGRESS:
       zenity_progress (results->data, results->progress_data);
       break;
@@ -1194,6 +1282,12 @@ zenity_parse_options_callback (poptContext              ctx,
 
       results->mode = MODE_LIST;
       break;
+    case OPTION_NOTIFICATION:
+      if (results->mode != MODE_LAST)
+        zenity_error (NULL, ERROR_DIALOG);
+
+      results->mode = MODE_NOTIFICATION;
+      break;
     case OPTION_PROGRESS:
       if (results->mode != MODE_LAST)
         zenity_error (NULL, ERROR_DIALOG);
@@ -1251,13 +1345,13 @@ zenity_parse_options_callback (poptContext              ctx,
     case OPTION_PROGRESSTEXT: 
     case OPTION_LISTTEXT: 
     case OPTION_WARNINGTEXT: 
-      
+    case OPTION_NOTIFICATIONTEXT:  
       /* FIXME: This is an ugly hack because of the way the poptOptions are 
        * ordered above. When you try and use an --option more than once 
        * parse_options_callback gets called for each option. Suckage 
        */
 
-      if (parse_option_text > 7) 
+      if (parse_option_text > 8) 
         zenity_error ("--text", ERROR_DUPLICATE); 
      
       switch (results->mode) { 
@@ -1284,6 +1378,10 @@ zenity_parse_options_callback (poptContext              ctx,
           results->tree_data->dialog_text = g_locale_to_utf8 (g_strcompress (arg),
 						              -1, NULL, NULL, NULL);
           break; 
+        case MODE_NOTIFICATION:
+          results->notification_data->notification_text = g_locale_to_utf8 (g_strcompress (arg),
+                                                                            -1, NULL, NULL, NULL);
+          break;
         default: 
           zenity_error ("--text", ERROR_SUPPORT); 
       } 
@@ -1396,6 +1494,18 @@ zenity_parse_options_callback (poptContext              ctx,
 
       results->file_data->multi = TRUE;
       break;
+    case OPTION_DIR:
+      if (results->mode != MODE_FILE)
+        zenity_error ("--directory", ERROR_SUPPORT);
+
+      results->file_data->directory = TRUE;
+      break;
+    case OPTION_SAVE:
+      if (results->mode != MODE_FILE)
+        zenity_error ("--save", ERROR_SUPPORT);
+
+      results->file_data->save = TRUE;
+      break;
     case OPTION_COLUMN: 
       if (results->mode != MODE_LIST) 
         zenity_error ("--column", ERROR_SUPPORT); 
@@ -1456,6 +1566,12 @@ zenity_parse_options_callback (poptContext              ctx,
 
       results->progress_data->autoclose = TRUE;
       break;
+    case OPTION_PRINTCOLUMN:
+      if (results->mode != MODE_LIST)
+        zenity_error ("--print-column", ERROR_SUPPORT);
+
+      results->tree_data->print_column = g_strdup (arg);
+      break;
     case OPTION_ABOUT: 
       if (results->mode != MODE_LAST) 
         zenity_error (NULL, ERROR_DIALOG); 
